/****************************************************************************
   Program:     $Id: spoofer.c,v 1.10 2006/01/24 21:28:41 rbeverly Exp $
   Author:      Rob Beverly <rbeverly@mit.edu>
   Date:        $Date: 2006/01/24 21:28:41 $
   Description: Spoofer - generate spoofed packets
****************************************************************************/
#include "spoof.h"


/* Fill buffer with the spoofed packet */
int craftPacket(char *buf, char *payload, int payloadlen,
             struct sockaddr_in *src, struct sockaddr_in *dst, u_char ttl) {
    struct udpiphdr *ui;
    struct ip *ip;
    int packetlen;

    memset(buf, 0, BUFSIZE);
    ip = (struct ip *)buf;
    ui = (struct udpiphdr *)buf;
    packetlen = payloadlen + sizeof(struct udpiphdr);

    /* Copy payload */
    memcpy((buf + sizeof(struct udpiphdr)), payload, payloadlen);

    /* Create Pseudo-Header First */
    ui->ui_pr = IPPROTO_UDP;
    ui->ui_len = htons(payloadlen + sizeof(struct udphdr));
    ui->ui_src = src->sin_addr;
    ui->ui_dst = dst->sin_addr;
    ui->ui_sport = src->sin_port;
    ui->ui_dport = dst->sin_port;
    ui->ui_ulen = ui->ui_len;
    if ((ui->ui_sum = in_cksum((u_short *) ui, packetlen)) == 0)
        ui->ui_sum = 0xffff;

    ip->ip_v = IPVERSION;
    ip->ip_hl = sizeof(struct ip) >> 2;
    ip->ip_tos = 0;

    /* FreeBSD raw sockets expect host not network order */
#ifdef _FBSD
    ip->ip_len = packetlen;
    ip->ip_off = IP_DF;
#else
    ip->ip_len = htons(packetlen);
    ip->ip_off = htons(IP_DF);
#endif
    ip->ip_id = rand() % 32768;
    ip->ip_sum = 0;
    ip->ip_ttl = ttl;

    if (DEBUG) {
        printf("Complete packet:\n");
        dumpPacket((unsigned char *)ip, packetlen);
    }
    return (packetlen);
}


/* Build Spoofer Report */
void craftReport(spoof_t *report, unsigned short unused, 
                unsigned long addr, char *seqno) {
#ifdef SPOOFER_BIG_ENDIAN
    report->ver = byte16Swap(VERSION);
    report->unused = byte16Swap(unused);
#else
    report->ver = VERSION;
    report->unused = unused;
#endif
    report->addr = addr;
    strncpy(report->seqno, seqno, SEQNOSIZE);
}


int main(int argc, char **argv) {
#ifdef _WIN32
    BOOL on = TRUE;
    WSADATA wsd;
    int ret;
#else
    int on = 1;
#endif
    struct sockaddr_in src, dst;
    struct in_addr myaddress, tmpaddr;
    struct in_addr a[ADDRS + SCANSPACE];
    int numAddrs = ADDRS;
    unsigned char *seqno[ADDRS + SCANSPACE];
    unsigned char buf[BUFSIZE];
    unsigned int gap;
    char arg[SMALLBUF], sessionkey[SMALLBUF];
    char tracebuf[BIGBUF];
    spoof_t report;
    char *icmpkey;
    unsigned long sessionid = 0;
    unsigned long dist = 0;
    int spoofsock, ctrlsock, udpsock;
    int status = VERSION;
    int packetlen;
    int i, j;

    a[0].s_addr = inet_addr("1.2.3.4");      /* valid, not in global BGP table */
    a[1].s_addr = inet_addr("172.16.1.100"); /* invalid, RFC1918 addr */
    a[2].s_addr = inet_addr("6.1.2.3");      /* valid, in global BGP table */

    printf(">> Spoofing Tester v0.%d\n", VERSION);
    printf(">> Rob Beverly <rbeverly at mit dot edu>\n");
    printf(">> More information: http://%s\n", REPORT_HOST);
    printf(">>\n");

#ifndef _WIN32
    if (getuid() != 0) {
        printf("** Must be run as root!\n");
        exit(-1);
    }
#endif

#ifdef _WIN32
    /* Windows socket API startup */
    if (WSAStartup(MAKEWORD(2, 2), &wsd) != 0) {
        winfatal("** WSAStartup() failed");
    }
#endif

    /* Establish TCP control connection */
    src.sin_family = AF_INET;
    dst.sin_family = AF_INET;
    dst.sin_addr.s_addr = inet_addr(SERVER);
    src.sin_port = htons(SRC_PORT);
    dst.sin_port = htons(REPORT_PORT);
#ifdef _WIN32
    ctrlsock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (ctrlsock == INVALID_SOCKET) {
        winfatal("** WSASocket() failed: %d\n", WSAGetLastError());
    }
    ret = connect(ctrlsock, (SOCKADDR *) &dst, sizeof(dst));
    if (ret == SOCKET_ERROR) {
        winfatal("** connect failed: %d\n", WSAGetLastError());
    }
#else
    if ((ctrlsock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
        perror("create TCP socket");
        exit(-1);
    }
    if ((connect(ctrlsock, (struct sockaddr *) &dst, sizeof(dst))) < 0) {
        perror("connect");
        exit(-1);
    }
#endif

    /* Create non-spoofing UDP socket for sanity check */
#ifdef _WIN32
    udpsock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (udpsock == INVALID_SOCKET) {
        winfatal("** WSASocket() failed: %d\n", WSAGetLastError());
    }
#else
    if ((udpsock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
        perror("create udp socket");
        exit(-1);
    }
#endif

    /* Force TTL for non-spoofed UDP packets to measure hop distance */
    on = TTL_OUT;
#ifdef _WIN32
    ret = setsockopt(udpsock, IPPROTO_IP, IP_TTL, (char *)&on, sizeof(on));
    if (ret == SOCKET_ERROR) {
        winfatal("** setsockopt(IP_TTL) failed: %d\n", WSAGetLastError());
    }
#else
    if ((setsockopt(udpsock, IPPROTO_IP, IP_TTL, (char *)&on, sizeof(on))) < 0) {
        perror("setsockopt IP_TTL");
        exit(-1);
    }
#endif
    on = 1;

    /* Create UDP spoofing socket */
#ifdef _WIN32
    spoofsock = socket(AF_INET, SOCK_RAW, IPPROTO_UDP);
    if (spoofsock == INVALID_SOCKET) {
        winfatal("** WSASocket() failed: %d\n", WSAGetLastError());
    }
    ret = setsockopt(spoofsock, IPPROTO_IP, IP_HDRINCL, (char *)&on, sizeof(on));
    if (ret == SOCKET_ERROR) {
        winfatal("** setsockopt(IP_HDRINCL) failed: %d\n", WSAGetLastError());
    }
#else
    if ((spoofsock = socket(AF_INET, SOCK_RAW, IPPROTO_UDP)) < 0) {
        perror("create raw socket");
        exit(-1);
    }
    if ((setsockopt(spoofsock, IPPROTO_IP, IP_HDRINCL, (char *)&on, sizeof(on))) < 0) {
        perror("setsockopt IP_HDRINCL");
        exit(-1);
    }
#endif

    /* Say Hello. */
    sprintf(buf, "HELLO %s %d\n", OS, VERSION);
    Writen(ctrlsock, buf, strlen(buf));
    if (Readline(ctrlsock, buf, BUFSIZE) == 0)
        printf("** server terminated prematurely");
    if (sscanf(buf, "%80s %lu", arg, (unsigned long *)&sessionid) == 2)
        printf(">> Server response: %s %lu\n", arg, sessionid);

    /* Play nicely. */
    if (strstr(arg, "FORBIDDEN") != NULL) {
        printf(">> Client forbidden by server.  Bye.\n");
        goto endtest;
    }
    srand(time(NULL));

    /* Get my IP; populate adjacent netblock array */
    sprintf(buf, "WHOAMI %s %d\n", OS, VERSION);
    Writen(ctrlsock, buf, strlen(buf));
    if (Readline(ctrlsock, buf, BUFSIZE) == 0)
        printf("** server terminated prematurely");
    if (sscanf(buf, "%80s %lu", arg, (unsigned long *)&(tmpaddr.s_addr)) == 2) {
#ifdef SPOOFER_BIG_ENDIAN
        myaddress.s_addr = (unsigned long) byte32Swap(tmpaddr.s_addr);
#else
        myaddress = tmpaddr;
#endif
        if (DEBUG) {
            printf("Got myaddr: %s\n", inet_ntoa(myaddress));
            printf("RFC1918?: %d\n", checkRFC1918(&myaddress));
        }
        /* only scan adjacent address prefixes if not RFC1918 */
        if (checkRFC1918(&myaddress) != 1) {
            for (i = 0; i < SCANSPACE; i++) {
                tmpaddr = myaddress;
                if (ntohl(myaddress.s_addr) & (1 << i))
                    tmpaddr.s_addr = htonl(ntohl(myaddress.s_addr) - (1 << i));
                else
                    tmpaddr.s_addr = htonl(ntohl(myaddress.s_addr) + (1 << i));
                    a[i + ADDRS] = tmpaddr;
            }
            numAddrs = ADDRS + SCANSPACE;
        }
    }

    /* Send UDP packets to the DNS port of spoofer server */
    dst.sin_port = htons(SERV_PORT);

    /* Make certain non-spoofed packets can reach the server */
    craftReport(&report, sessionid, myaddress.s_addr, NONCE);
    printf(">> Source %d non-spoofed packets...\n", DUPLICATES);
    for (j = 0; j < DUPLICATES; j++) {
        /* random interpacket delay 0-0.5s */
        gap = (unsigned int)((double)rand() / ((double)RAND_MAX + 1) * 500);
#ifdef _WIN32
        ret = sendto(udpsock, (unsigned char *)&report, sizeof(report), 0, (SOCKADDR *) &dst, sizeof(dst));
        if (ret == SOCKET_ERROR) {
            printf("** udp sendto() failed: %d\n", WSAGetLastError());
        }
        Sleep(gap);
#else
        if ((sendto(udpsock, (unsigned char *)&report, sizeof(report), 0, (struct sockaddr *) &dst, sizeof(dst))) < 0) {
            perror("udp sendto");
        }
        usleep(gap * 1000);
#endif
    }

    /* malloc space for array of sequence numbers */
    for (i = 0; i < numAddrs; i++) {
        seqno[i] = malloc(sizeof(char) * (SEQNOSIZE + 1));
    }

    /* Send spoofed packets */
    for (i = 0; i < numAddrs; i++) {
        src.sin_addr = a[i];
        genSequence(seqno[i], SEQNOSIZE);
        craftReport(&report, 0, src.sin_addr.s_addr, seqno[i]);
        if (DEBUG) {
            printf("Spoof attempt sequence number:\n");
            dumpPacket(report.seqno, SEQNOSIZE);
        }
        packetlen = craftPacket(buf, (char *)&report, PAYLOADSIZE, &src, &dst, TTL_OUT);
        printf(">> Source %d spoofed packets (IP: %s)...\n",
               DUPLICATES, inet_ntoa(a[i]));
        for (j = 0; j < DUPLICATES; j++) {
            /* random interpacket delay 0-0.5s */
            gap = (unsigned int)((double)rand() / ((double)RAND_MAX + 1) * 500);
#ifdef _WIN32
            ret = sendto(spoofsock, buf, packetlen, 0, (SOCKADDR *) &dst, sizeof(dst));
            if (ret == SOCKET_ERROR) {
                if (WSAGetLastError() == 10004) {
                    printf("%s", WIN_SEND_ERR);
                    /* bail out early, setting all seq numbers to fail */
                    status = WINFAIL;
                    goto endtest;
                } else {
                    printf("** sendto() failed: %d\n", WSAGetLastError());
                    strcpy(seqno[i], "SENDTOFAIL");
                }
                /* don't try to send duplicates on error */
                j += DUPLICATES;
            }
            Sleep(gap);
#else
            if ((sendto(spoofsock, buf, packetlen, 0, (struct sockaddr *) &dst, sizeof(dst))) < 0) {
                perror("sendto");
                strcpy(seqno[i], "SENDTOFAIL");
            }
            usleep(gap * 1000);
#endif
        }
    }

    /* Get my IP hop distance from the server */
    sprintf(buf, "DISTANCE %s %d\n", OS, VERSION);
    Writen(ctrlsock, buf, strlen(buf));
    if (Readline(ctrlsock, buf, BUFSIZE) == 0)
        printf("** server terminated prematurely");
    if (sscanf(buf, "%80s %lu", arg, (unsigned long *)&dist) == 2) {
        printf(">> Server response: %s %lu\n", arg, dist);
    } else {
        printf("** bogus distance response");
        dist = 0;
    }

    /* Send TTL-limited reverse traceroute spoofed packets */
    src.sin_addr.s_addr = inet_addr(SERVER);
    dst.sin_addr.s_addr = inet_addr(VALIDDST);
    /* we encode the session ID into the UDP source port */
    src.sin_port = htons(sessionid);
    for (i = 1; i <= dist; i++) {
        /* the TTL of the packet will be encoded as the size of the spoofed
         * UDP datagram.  TTL = UDPlength - 8 */
        icmpkey = calloc(i, sizeof(char));
        packetlen = craftPacket(buf, icmpkey, i, &src, &dst, i);
        printf(">> Source %d reverse traceroute spoofed packets (TTL: %d)...\n",
               DUPLICATES, i);
        for (j = 0; j < DUPLICATES; j++) {
            /* random interpacket delay 0-0.5s */
            gap = (unsigned int)((double)rand() / ((double)RAND_MAX + 1) * 500);
#ifdef _WIN32
            ret = sendto(spoofsock, buf, packetlen, 0, (SOCKADDR *) &dst, sizeof(dst));
            if (ret == SOCKET_ERROR) {
                printf("** sendto() failed: %d\n", WSAGetLastError());
                j += DUPLICATES;
            }
            Sleep(gap);
#else
            if ((sendto(spoofsock, buf, packetlen, 0, (struct sockaddr *) &dst, sizeof(dst))) < 0) {
                perror("sendto");
            }
            usleep(gap * 1000);
#endif
        }
    }

    /* Send UDP packets to the DNS port of spoofer server */
    dst.sin_addr.s_addr = inet_addr(SERVER);
    dst.sin_port = htons(SERV_PORT);
    /* We send this final round of non-spoofed UDP packets to determine 
       if the TTL (hop-count) changed during the test */
    craftReport(&report, sessionid, myaddress.s_addr, NONCE);
    printf(">> Source %d non-spoofed packets...\n", DUPLICATES);
    for (j = 0; j < DUPLICATES; j++) {
        /* random interpacket delay 0-0.5s */
        gap = (unsigned int)((double)rand() / ((double)RAND_MAX + 1) * 500);
#ifdef _WIN32
        ret = sendto(udpsock, (unsigned char *)&report, sizeof(report), 0, (SOCKADDR *) &dst, sizeof(dst));
        if (ret == SOCKET_ERROR) {
            printf("** udp sendto() failed: %d\n", WSAGetLastError());
        }
        Sleep(gap);
#else
        if ((sendto(udpsock, (unsigned char *)&report, sizeof(report), 0, (struct sockaddr *) &dst, sizeof(dst))) < 0) {
            perror("udp sendto");
        }
        usleep(gap * 1000);
#endif
    }

    /* Confer with server to get results */
    printf(">>\n");
    printf(">> Checking spoofing result...\n");
    sprintf(buf, "SENT %d %d\n", numAddrs, VERSION);
    Writen(ctrlsock, buf, strlen(buf));
    if (Readline(ctrlsock, buf, BUFSIZE) == 0)
        printf("** server terminated prematurely");
    printf(">> Server response: %s", buf);
    for (i = 0; i < numAddrs; i++) {
        memset(buf, 0, BUFSIZE);
#ifdef SPOOFER_BIG_ENDIAN
        sprintf(buf, "SPOOF %s %lu\n", seqno[i], byte32Swap(a[i].s_addr));
#else
        sprintf(buf, "SPOOF %s %lu\n", seqno[i], (unsigned long)a[i].s_addr);
#endif
        Writen(ctrlsock, buf, strlen(buf));
        if (Readline(ctrlsock, buf, BUFSIZE) == 0)
            printf("** server terminated prematurely");
        printf(">> Server response: %s", buf);
    }

    /* run and send trace */
    findtr(buf);
    runtrace(buf, SERVER, tracebuf);
    sprintf(buf, "TRACE %s %d\n", OS, VERSION);
    Writen(ctrlsock, buf, strlen(buf));
    if (Readline(ctrlsock, buf, BUFSIZE) == 0)
        printf("** server terminated prematurely");
    printf(">> Server response: %s", buf);
    Writen(ctrlsock, tracebuf, strlen(tracebuf));

endtest:
    /* end and cleanup */
    sprintf(buf, "DONE %s %d\n", OS, status);
    Writen(ctrlsock, buf, strlen(buf));
    if (Readline(ctrlsock, buf, BUFSIZE) == 0)
        printf("** server terminated prematurely");
    if (sscanf(buf, "%80s %80s", arg, sessionkey) == 2)
        printf(">> Server response: %s %s\n", arg, sessionkey);

    close(udpsock);
    close(spoofsock);
    close(ctrlsock);

    snprintf(buf, SMALLBUF*2, "http://%s/report.php?sessionkey=%s\n",
            REPORT_HOST, sessionkey);
    printf("\nTest Complete.\n");
    printf("Your test results:\n\t%s\n", buf);
#ifdef _WIN32
    /* spawn an IE window with the results */
    snprintf(buf, SMALLBUF*2, "c:\\progra~1\\intern~1\\iexplore http://%s/report.php?sessionkey=%s\n",
            REPORT_HOST, sessionkey);
    system(buf);
    winpause("Press Enter to Exit.");
#endif
    exit(0);
}
